/**
 * Controller class for interacting with a HAP device over HTTP.
 */
/// <reference types="node" />
import { EventEmitter } from 'events';
import HttpConnection from './http-connection';
import { PairingData } from '../../protocol/pairing-protocol';
import { TLV } from '../../model/tlv';
export interface GetCharacteristicsOptions {
    meta?: boolean;
    perms?: boolean;
    type?: boolean;
    ev?: boolean;
}
export default class HttpClient extends EventEmitter {
    deviceId: string;
    private address;
    private port;
    private pairingProtocol;
    private _pairingConnection?;
    /**
     * Initialize the HttpClient object.
     *
     * @param {string} deviceId - ID of the device
     * @param {string} address - IP address of the device
     * @param {number} port - HTTP port
     * @param {Object?} pairingData - existing pairing data
     */
    constructor(deviceId: string, address: string, port: number, pairingData?: PairingData);
    /**
     * Get the data (keys) that needs to be stored long-term.
     *
     * @returns {Object} Object containing the keys that should be stored.
     */
    getLongTermData(): PairingData | null;
    /**
     * Run the identify routine on a device.
     *
     * This can only be done before pairing.
     *
     * @returns {Promise} Promise which resolves if identify succeeded.
     */
    identify(): Promise<void>;
    /**
     * Begins the pairing process. For devices with random pins, this
     * will cause it to show the pin on the screen.
     *
     * @param {PairMethods} [pairMethod] - Method to use for pairing, default is PairSetupWithAuth
     * @param {PairingTypeFlags} [pairFlags] - Flags to use for Pairing for PairSetup
     * @returns {Promise} Promise which resolves to opaque
     * pairing data when complete.
     */
    startPairing(pairMethod?: number, pairFlags?: number): Promise<TLV>;
    /**
     * Finishes a pairing process that began with startPairing()
     *
     * @param {Object} pairingData - The pairing data returned from startPairing()
     * @param {string} pin - The pairing PIN
     * @returns {Promise} Promise which resolvew when pairing is complete.
     */
    finishPairing(pairingData: TLV, pin: string): Promise<void>;
    /**
     * Attempt to pair with a device.
     *
     * @param {string} pin - The pairing PIN
     * @param {PairMethods} [pairMethod] - Method to use for pairing, default is PairSetupWithAuth
     * @param {PairingTypeFlags} [pairFlags] - Flags to use for Pairing for PairSetup
     * @returns {Promise} Promise which resolves when pairing is complete.
     */
    pairSetup(pin: string, pairMethod?: number, pairFlags?: number): Promise<void>;
    /**
     * Method used internally to generate session keys for a connection.
     *
     * @param {Object} connection - Existing HttpConnection object
     * @returns {Promise} Promise which resolves to the generated session keys.
     */
    private _pairVerify;
    /**
     * Unpair the controller from a device.
     *
     * @param {string} identifier - Identifier of the controller to remove
     * @returns {Promise} Promise which resolves when the process completes.
     */
    removePairing(identifier: string): Promise<void>;
    /**
     * Add a pairing to a device.
     *
     * @param {string} identifier - Identifier of new controller
     * @param {Buffer} ltpk - Long-term public key of the new controller
     * @param {boolean} isAdmin - Whether or not the new controller is an admin
     * @returns {Promise} Promise which resolves when the process is complete.
     */
    addPairing(identifier: string, ltpk: Buffer, isAdmin: boolean): Promise<void>;
    /**
     * List the pairings on a device.
     *
     * @returns {Promise} Promise which resolves to the final TLV when the process
     *                    is complete.
     */
    listPairings(): Promise<TLV>;
    /**
     * Get the accessory attribute database from a device.
     *
     * @returns {Promise} Promise which resolves to the JSON document.
     */
    getAccessories(): Promise<Record<string, unknown>>;
    /**
     * Read a set of characteristics.
     *
     * @param {string[]} characteristics - List of characteristics ID to get
     * @param {Object?} options - Options dictating what metadata to fetch
     * @returns {Promise} Promise which resolves to the JSON document.
     */
    getCharacteristics(characteristics: string[], options?: GetCharacteristicsOptions): Promise<Record<string, unknown>>;
    /**
     * Modify a set of characteristics.
     *
     * @param {Object} characteristics - Characteristic IDs to set, id -> val
     * @returns {Promise} Promise which resolves to the JSON document.
     */
    setCharacteristics(characteristics: Record<string, unknown>): Promise<Record<string, unknown>>;
    /**
     * Subscribe to events for a set of characteristics.
     *
     * @param {String[]} characteristics - List of characteristic IDs to subscribe
     *                   to
     * @returns {Promise} Promise which resolves to the HttpConnection object.
     */
    subscribeCharacteristics(characteristics: string[]): Promise<HttpConnection>;
    /**
     * Unsubscribe from events for a set of characteristics.
     *
     * @param {String[]} characteristics - List of characteristic IDs to
     *                   unsubscribe from
     * @param {Object} connection - Existing HttpConnection object
     * @returns {Promise} Promise which resolves when the procedure is done.
     */
    unsubscribeCharacteristics(characteristics: string[], connection: HttpConnection): Promise<void>;
}
//# sourceMappingURL=http-client.d.ts.map